#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#  include <config_any.h>
#endif

#include <utils.h>
#include <gtkmrcomune.h>

/*****************************************************************/
/*                   H E L P   O N    K E Y B O A R D            */
/*****************************************************************/

#define NUM_FIELDS 5
static const char *titles[] = { "Category", "Command", "Keys", 
                          "Description", "Icon" };
static GtkWidget *khelp_dlg = NULL;

GtkWidget *create_keyboard_help(GtkWidget *main_window, std::string fhelp)
{
   GtkWidget *dialog, *vbox, *clist, *button, *scrwin;
   int i, num;
//   nomefile fhelp;
   char *fields[NUM_FIELDS];
   FILE *fp;
   char rigar[MAX_LEN];
   GtkWidget *label;
   GtkListStore *store;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
   GtkTreeIter iter;
   GdkPixbuf *icobuf;
   GType *types = g_new0(GType, NUM_FIELDS);

   for(i=0; i<NUM_FIELDS-1; i++)
     types[i] = G_TYPE_STRING;
   types[NUM_FIELDS-1] = GDK_TYPE_PIXBUF;

   dialog = gtk_dialog_new();
   gtk_window_set_title (GTK_WINDOW (dialog), "Help on keyboard & mouse");
   gtk_window_set_default_size(GTK_WINDOW(dialog), 300, 300);

   vbox = gtk_dialog_get_content_area(GTK_DIALOG (dialog));

   scrwin = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwin), GTK_POLICY_NEVER,
              GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(vbox), scrwin);
#ifdef IN_GTK3
   gtk_widget_set_vexpand(scrwin, TRUE);
#endif

   clist = gtk_tree_view_new();
   store = gtk_list_store_newv(NUM_FIELDS, types);
   renderer = gtk_cell_renderer_text_new ();
   for(i=0; i<NUM_FIELDS-1; i++)
   {
        column = gtk_tree_view_column_new_with_attributes (titles[i],
            renderer, "text", i, NULL);
        gtk_tree_view_column_set_clickable(column, FALSE);
        gtk_tree_view_append_column (GTK_TREE_VIEW(clist), column); 
        label = LABEL_BOLD(titles[i], "red", 1.2f);
        gtk_widget_show (label);
        gtk_tree_view_column_set_widget(column, label);
   }
   renderer = gtk_cell_renderer_pixbuf_new ();
   column = gtk_tree_view_column_new_with_attributes (titles[NUM_FIELDS-1],
            renderer, "pixbuf", NUM_FIELDS-1, NULL);
   gtk_tree_view_column_set_clickable(column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW(clist), column); 
   label = LABEL_BOLD(titles[NUM_FIELDS-1], "red", 1.2f);
   gtk_widget_show (label);
   gtk_tree_view_column_set_widget(column, label);

   gtk_tree_view_set_model(GTK_TREE_VIEW(clist), GTK_TREE_MODEL (store));
   gtk_container_add(GTK_CONTAINER(scrwin), clist);

   button = myimage_button("   _OK", "ok.png");
   gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button, 1);
   gtk_widget_set_can_default (button,  TRUE);
   gtk_widget_grab_default (button);
   gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
       GTK_BUTTONBOX_SPREAD);
   (void) g_signal_connect_swapped(G_OBJECT(button), "clicked",
	  G_CALLBACK(gtk_widget_hide), dialog);
   (void) g_signal_connect_swapped(G_OBJECT(dialog), "delete_event",
        G_CALLBACK(gtk_button_clicked), G_OBJECT(button)); 
   (void) g_signal_connect (G_OBJECT (dialog), "delete_event",
                      G_CALLBACK(true_cb), NULL);


/* riempiamo la lista */
   fp = fOpen(fhelp.c_str(), "r");
   if(!fp)
   {
      (void) MessageDialog(main_window, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
            "Error in open file:", fhelp.c_str());
      return NULL;
   }

   while(fgets(rigar,MAX_LEN,fp) != NULL)
   {
      if(strlen(rigar) > 2)
      {
      std::string linea = filtra(rigar, "\n");
      num = get_str_tokens (linea.c_str(), ",", NUM_FIELDS, fields);
      gtk_list_store_append (store, &iter);
      if(num == NUM_FIELDS)
      {
         icobuf = create_pixbuf(g_strstrip(fields[4]));
         gtk_list_store_set (store, &iter, 4, icobuf, -1);
      }
      for(i=0; i<NUM_FIELDS-1; i++)
        gtk_list_store_set (store, &iter, i, g_strstrip(fields[i]), -1);
      }
   }
   (void) fclose(fp);

   set_selection_mode(clist, GTK_SELECTION_NONE);
   return dialog;
}

void open_khelp (GtkWidget *main_window, std::string fhelp)
{
   if(khelp_dlg == NULL)
      khelp_dlg = create_keyboard_help(main_window, fhelp);
   if(khelp_dlg)
   {
      gtk_window_set_transient_for(GTK_WINDOW(khelp_dlg), GTK_WINDOW(main_window));
      gtk_widget_show_all(khelp_dlg);
      gdk_window_raise (gtk_widget_get_window(khelp_dlg));
   }
}
