MODULE sinterface

implicit none

interface bij_atom
  module procedure bij_atoms, bij_atomv
end interface

interface bij_sir
  module procedure bij_sirs, bij_sirv
end interface

CONTAINS 

   subroutine fillatom(atom)
   USE Atomi
   USE atom_basic
   type(atom_type), dimension(:), intent(in) :: atom
   integer                                   :: i,j
!
   do i=1,nats
      do j=1,3
         xyzs(j,i) = atom(i)%xc(j)
      enddo
      nz(i) = atom(i)%get_nz()
      xyzs(4,i) = atom(i)%inte
      xyzs(5,i) = atom(i)%ocry
      occpch(i) = atom(i)%och
      bfac(i) = atom(i)%biso
      aLab(i) = atom(i)%lab
   enddo
!
   end subroutine fillatom

!----------------------------------------------------------------------

   subroutine atom_to_xyzs(atom,legm)
!
!  Copia atom e legm in variabili xyzs e iconn
!
   USE Atomi
   USE connect_mod
   type(atom_type), dimension(:), intent(in)                        :: atom
   type(bond_type), dimension(:), intent(in), allocatable, optional :: legm
   integer                                                          :: i,j
!
   nats = size(atom)
   do i=1,nats
      do j=1,3
         xyzs(j,i) = atom(i)%xc(j)
      enddo
      nz(i) = atom(i)%get_nz()
      xyzs(4,i) = atom(i)%inte
      xyzs(5,i) = atom(i)%ocry
      occpch(i) = atom(i)%och
      bfac(i) = atom(i)%biso                                                                    
      aLab(i) = atom(i)%lab
      bij(:,i) = bij_sir(atom(i)%bij)
   enddo
!
   if (present(legm)) call bond_to_iconn(legm,iconn)
!
   end subroutine atom_to_xyzs

!--------------------------------------------------------------------------------------
   
   subroutine xyzs_to_atom(atom)
   USE Atomi
   USE atom_type_util
   USE General, only: celem
   type(atom_type), intent(inout), dimension(:), allocatable :: atom
   call xyz_to_atom(atom,xyzs(:3,:nats),bfac,bij_atom(bij),xyzs(5,:),occpch,nz,aLab,xyzs(4,:),elem=celem)
   end subroutine xyzs_to_atom

!--------------------------------------------------------------------------------------

   subroutine update_model(atom,legm,id,redraw)
   USE variables
   USE atom_type_util
   USE connect_mod
   USE Molcom, only: jscreen
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   logical, intent(in), optional                          :: redraw
   integer, intent(in)                                    :: id
!
!  update xyzs,atm,lconn
   if (present(redraw)) then
       call update_model_old(atom,legm,redraw)
   else
       call update_model_old(atom,legm,(jscreen > 0))
   endif
!
   if (numatoms(atom) > 0) then
!
!      Update crystal phase 1
       call cryst(id)%set_atoms(atom)
       call cryst(id)%clear_constraints()   !FIX this updating constr. if you add/remove atoms
       call cryst(id)%clear_restraints()    !FIX this updating restra. if you add/remove atoms
       call cryst(id)%set_bonds(legm)
   endif
!
   end subroutine update_model

!--------------------------------------------------------------------------------------

   subroutine update_model_old(atom,legm,redraw) 
   USE variables
   USE atom_type_util
   USE connect_mod
   USE Atomi, only : nats, nz, nzo
   USE General, only: InfoOK
   USE Molcom, only : iPlotType   !, jscreen
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   logical, intent(in)                                    :: redraw
   integer :: iPlt, ifSingle, natsOld, localNat
!
!  aggiorna xyzs e iconn
   localNat = numatoms(atom)
   if (localNat > 0) then
       if (nats > 0) then
           if (localNat /= nats) then
               call AllocaAtomi(2)
               call AllocaModelRemo(2)
               nats = localNat; call AllocaAtomi(-1)
           endif
       else
           nats = localNat; call AllocaAtomi(-1)
       endif
       call atom_to_xyzs(atom,legm)
       nzo(1:nats) = nz(1:nats)
       natsOld = nats
!
!      aggiorna atm
       call copy_atoms(atm,atom)
!
!      aggiorna lconn
       call copy_bonds(lconn,legm)
!
!      gestisce gli eventuali idrogeni generati da Jav 
       call ManageJavHydro(natsOld)
!
!      aggiorna Model
       call PutModel
       InfoOK(5) = 1
!
!      ri-disegna
       if (redraw) then
           iPlt = iplotType
           ifSingle = 1
           call MolPlo(iPlt, ifSingle)
       endif
   endif
!
   end subroutine update_model_old

!--------------------------------------------------------------------------------------

   subroutine update_atm()
   USE Atomi
   USE variables
   USE atom_type_util
   USE connect_mod
   USE General, only: cell,celem
   USE unit_cell
   type(cell_type) :: cellt
!
   if (nats > 0) then
!
!      aggiorna atm
       call xyz_to_atom(atm,xyzs(:3,:nats),bfac,bij_atom(bij),xyzs(5,:),occpch,nz,aLab,xyzs(4,:),elem=celem)
!
!      aggiorna lconn
       cellt = set_cell_type(cell)
       call iconn_to_legm(iconn,lconn,atm,cellt%get_g())
   endif
!
   end subroutine update_atm

!--------------------------------------------------------------------------------------

   function spaceg_string() result(space)
   USE Symmetry
   USE strutil
   character(len=16) :: space
   write(space,'(16a1)')nSpaceG
   space(1:1) = upper(space(1:1))
   end function spaceg_string

!--------------------------------------------------------------------------------------

   logical function is_cell_defined_list()
   USE General, only:cell, InfoOK
!
!  InfoOK(1) == 1 significa che e' stata creata la lista 1
   is_cell_defined_list = all(cell(1:6) > 0.0) .and. InfoOK(1) == 1
!
   end function is_cell_defined_list

!--------------------------------------------------------------------------------------

   logical function is_space_defined_list()
   USE strutil
   USE General, only:InfoOK
   character(len=16) :: spg
   spg = spaceg_string()
   call s_filter(spg)
!
!  InfoOK(1) == 1 significa che e' stata creata la lista 1
   is_space_defined_list = len_trim(spg) /= 0 .and. InfoOK(2) == 1
   end function is_space_defined_list

!--------------------------------------------------------------------------------------

   subroutine set_cell_as_defined()
   USE General, only:cell, InfoOK
   if (all(cell(1:6) > 0.0)) InfoOK(1) = 1
   end subroutine set_cell_as_defined

!--------------------------------------------------------------------------------------

   subroutine set_space_as_defined()
   USE General, only:InfoOK
   if (len_trim(spaceg_string()) /= 0) InfoOK(2) = 1
   end subroutine set_space_as_defined

!--------------------------------------------------------------------------------------

   logical function is_cell_defined()
   USE General, only:cell
!
   is_cell_defined = all(cell(1:6) > 0.0)
!
   end function is_cell_defined

!--------------------------------------------------------------------------------------

   logical function is_space_defined()
   USE strutil
   character(len=16) :: spg
   spg = spaceg_string()
   call s_filter(spg)
   is_space_defined = len_trim(spg) /= 0 
   end function is_space_defined

!--------------------------------------------------------------------------------------

   subroutine create_cell()
   real, dimension(6) :: cella
   cella(1:3) = 10.0
   cella(4:6) = 90
   call set_cell(cella)
   end subroutine create_cell

!--------------------------------------------------------------------------------------

   subroutine set_cell(cella)
   USE General, only: cell,xgg
   USE unit_cell
   real, dimension(:), intent(in) :: cella
!
   cell(1:6) = cella(1:6)
   xgg = matrice_metrica(cell(1:6))
!
   end subroutine set_cell

!--------------------------------------------------------------------------------------

   subroutine save_cell_bin(unitbin,cell)
!
!  Save cell on binary format
!
   integer, intent(in)            :: unitbin
   real, dimension(:), intent(in) :: cell
!
   write(unitbin)cell(1:6)
!
   end subroutine save_cell_bin

!--------------------------------------------------------------------------------------

   subroutine read_cell_bin(unitbin,cell,err)
!
!  Read cell from binary file
!
   USE errormod
   integer, intent(in)             :: unitbin
   real, dimension(6), intent(out) :: cell
   integer                         :: ier
   type(error_type), intent(out)   :: err
!
   read(unitbin,iostat=ier) cell(1:6)
   if (ier == 0) then
       call set_cell(cell)
   else
       call err%set('Error on reading cell parameters')
   endif
!
   end subroutine read_cell_bin

!corr!--------------------------------------------------------------------------------------
!corr
!corr   subroutine save_space_bin(unitbin,space)
!corr   USE spginfom
!corr   integer, intent(in)           :: unitbin
!corr   type(spaceg_type), intent(in) :: space
!corr!   
!corr   write(unitbin)space
!corr!
!corr   end subroutine save_space_bin
!corr
!corr!--------------------------------------------------------------------------------------
!corr
!corr   subroutine read_space_bin(unitbin,err)
!corr   USE spginfom
!corr   USE errormod
!corr   integer, intent(in)           :: unitbin
!corr   type(error_type), intent(out) :: err
!corr   type(spaceg_type)             :: space
!corr   integer                       :: ier
!corr!
!corr   read(unitbin,iostat=ier)space
!corr   if (ier == 0) then
!corr       call set_spaceg_new(space)
!corr   else
!corr       call err%set('Error on reading space group')
!corr   endif
!corr!
!corr   end subroutine read_space_bin
!corr
!corr!--------------------------------------------------------------------------------------

   subroutine set_spaceg(space,ierspace)
   USE Symmetry
   USE General, only: spacegr
   USE spginfom
   character(len=*), intent(in)   :: space
   integer, intent(out), optional :: ierspace
   integer                        :: ishif
   integer, dimension(3)          :: kshif
   character(len=16)              :: spaceg
   integer                        :: ier
!
   ier = 0
   if (len_trim(space) > 0) then
       spaceg(:) = space
       call StandardSpg(spaceg,ier)
       if (ier == 0) then
           ishif = 0
           kshif(:) = 0
           call insym(ishif,kshif,ier)
           totop = pts * float( ( icent + 1 ) * nsym )     ! era nella yfrm02
           call xspinb()  
           spacegr = init_spaceg_type(space) ! set space group in structure spacegr
       endif
   endif
       !write(0,*)'NSYM=',nsym,nsymtot,latt,jsys
   if (present(ierspace)) ierspace = ier
!
   end subroutine set_spaceg

!--------------------------------------------------------------------------------------

   subroutine set_spaceg_new(spg)
   USE spginfom
   USE Symmetry
   USE General, only:cell, spacegr
   type(spaceg_type), intent(in) :: spg
   integer                       :: i,j
   real, dimension(13) :: celltmp
!
   spacegr = spg   ! save spacegroup in a global variable
!
!  Symbol
   if (len_trim(spg%symbol_xhm) > 0) then
       read(spg%symbol_xhm,'(16a1)') nSpaceG
   elseif (len_trim(spg%symbol_alt(1)) > 0) then
       read(spg%symbol_alt(1),'(16a1)') nSpaceG
   endif
!
!  Numero di operatori e centratura
   nsymTot = spg%nsymop
   totop = spg%nsymop
   pts = spg%ncoper
   ncentr = spg%ncoper
   if (spg%symcent == 1) then
       icent = 1
       nsym = spg%nsymop_prim / 2
   else
       icent = 0
       nsym = spg%nsymop_prim
   endif
!
!  Lattice code
   select case(spg%lattyp)
      case ('P'); latt = 1
      case ('A'); latt = 2
      case ('B'); latt = 3
      case ('C'); latt = 4
      case ('I'); latt = 5
      case ('F'); latt = 6
      case ('R'); latt = 7
   end select
!
!  Crystal system
   select case (spg%csys_code)
     case (CS_Triclinic); jsys = 1
     case (CS_Monoclinic); jsys = 2
     case (CS_Orthorhombic); jsys = 3
     case (CS_Tetragonal); jsys = 4
     case (CS_Hexagonal); jsys = 5
     case (CS_Cubic); jsys = 6
     case (CS_Trigonal)
       if (index(spg%symbol_xhm,':R') > 0 .or. uc_is_rhombohedral(cell,0.01)) then
           jsys = 7
       else
           jsys = 5
       endif
   end select
!   
!  Operatori e stringhe
          !write(0,*)'nsymtot=',nsymtot,spg%num
   do i=1,nsymTot
      kmat(i,:,:) = spg%symop(i)%rot
      tmat(i,:) = spg%symop(i)%trn
      SymmStrings(i) = spg%symopstr(i)
   enddo
   nSymmStrings = spg%nsymop
!   if (icent == 1) call CheckCentro(nsym)    ! serve????? contiene la xspinb che serve
   if (nsym <= 24) then  ! se nsym > 24 (es. F -4 2 3) supero la dim di ismat/tsmat
       call musym()    ! ismat, tsmat 
!corr       call Preoper()  ! server
       call InitOper()
   else  ! dare un warning
   endif
!
!  Direzione polare
   kpol = 0
   if (spg%polarx) vpol(1) = 1
   if (spg%polary) vpol(2) = 1
   if (spg%polarz) vpol(3) = 1
   kpol=100*vpol(1)+10*vpol(2)+vpol(3)
!
!  Origini permesse
   OkOrig = 1
   nOrigRoxi = spg%norig_ind
          !write(0,*)'norig=',nOrigRoxi,spg%norig
   do i=1,nOrigRoxi
      tabo(i,:) = spg%orig_ind(:,i)
      if(spg%polarx .and. spg%polary .and. spg%polarz) then
          tabo(i,:3) = 128
      else if( spg%polarx .and. spg%polary)    then
          tabo(i,1) = 128
          tabo(i,2) = 128
      else if( spg%polarx .and. spg%polarz)    then
          tabo(i,1) = 128
          tabo(i,3) = 128
      else if( spg%polary .and. spg%polarz)    then
          tabo(i,2) = 128
          tabo(i,3) = 128
      else if( spg%polarx)    then
          tabo(i,1) = 256
      else if( spg%polary)    then
          tabo(i,2) = 256
      else if( spg%polarz)    then
          tabo(i,3) = 256
      end if
   enddo
!
!  per xyzmax   serve???
   do i=1,nOrigRoxi
      do j=1,3
         if (tabo(i,j) > xyzmax(j)) then
             xyzmax(j) = tabo(i,j)
         endif
      end do
   end do
   do j=1,3
      if (xyzmax(j) < 0.6) then
          xyzmax(j) = 2.0
      elseif (xyzmax(j) < 0.7) then
          xyzmax(j) = 32.0   !  caso 1/3, 2/3 
      elseif (xyzmax(j) < 0.9) then
          xyzmax(j) = 32.0   !  caso 1/6, 5/6 
      elseif (xyzmax(j) < 1.0) then
          xyzmax(j) = 4.0
      endif
   end do
   celltmp(1:9) = cell
   call CheckRhombAx(nSpaceG, celltmp, ifRhombAx)
   call xspinb()
      !write(0,*)'lattice=',trim(spg%lattyp),latt
!!!???   if3den = ?????
!!!???   call calmod(ngc,nbr,ncs,sem,nse)
!
   end subroutine set_spaceg_new

!--------------------------------------------------------------------------------------
 
   function bij_atoms(bij_sir)
!
!  Convert sir bij in standard notation bij
!
   real, dimension(6) :: bij_sir
   real, dimension(6) :: bij_atoms
!
   bij_atoms(1) = bij_sir(1)    ! b11
   bij_atoms(2) = bij_sir(4)    ! b22
   bij_atoms(3) = bij_sir(6)    ! b33
   bij_atoms(4) = 0.5*bij_sir(2)    ! b12
   bij_atoms(5) = 0.5*bij_sir(3)    ! b13
   bij_atoms(6) = 0.5*bij_sir(5)    ! b23
!
   end function bij_atoms

!--------------------------------------------------------------------------------------

   function bij_atomv(bij_sir)
!
!  Convert sir bij in standard notation bij
!
   real, dimension(:,:)               :: bij_sir
   real, dimension(6,size(bij_sir,2)) :: bij_atomv
   integer                            :: i
!
   do i=1,size(bij_sir,2)
      bij_atomv(:,i) = bij_atoms(bij_sir(:,i))
   enddo
!
   end function bij_atomv

!--------------------------------------------------------------------------------------

   function bij_sirs(bij_atom)
!
!  Convert standard notation bij in sir bij
!
   real, dimension(6) :: bij_sirs
   real, dimension(6) :: bij_atom
!
   bij_sirs(1) = bij_atom(1)  ! b11
   bij_sirs(4) = bij_atom(2)  ! b22
   bij_sirs(6) = bij_atom(3)  ! b33
   bij_sirs(2) = 2.0*bij_atom(4)  ! b12
   bij_sirs(3) = 2.0*bij_atom(5)  ! b13
   bij_sirs(5) = 2.0*bij_atom(6)  ! b23
!
   end function bij_sirs

!--------------------------------------------------------------------------------------

   function bij_sirv(bij_atom)
!
!  Convert sir bij in standard notation bij
!
   real, dimension(:,:)                :: bij_atom
   real, dimension(6,size(bij_atom,2)) :: bij_sirv
   integer                             :: i
!
   do i=1,size(bij_atom,2)
      bij_sirv(:,i) = bij_sirs(bij_atom(:,i))
   enddo
!
   end function bij_sirv

!--------------------------------------------------------------------------------------

   subroutine set_symmcard(icen,lat,nsymc)
   USE Symmetry, only:nsym,latt,icent,ncentr,pts
   integer, intent(in) :: icen,lat,nsymc
   integer             :: ier
!
   nsym = nsymc
   latt = lat
   icent = icen
   select case (latt)
     case (1)
       pts = 1
     case (2:5)
       pts = 2
     case (6)
       pts = 4
     case (7)
       pts = 3
     case default
       pts = 0
   end select
   ncentr = pts
   call ManageSymmCards(ier)
   call xspinb()
!
   end subroutine set_symmcard

END MODULE sinterface
