#include <algorithm>
#include "draw_contour2d.h"
#include "cont_dlg1.h"
#include "cont_dlg2.h"
#include "cont_dlg3.h"

using namespace std;
static const char *layers[] = {"A", "B", "C", "X", "Y", "Z"};
string getFilePng(GLWindow *gl);
GdkPixbuf * fromCairoToPixbuf (cairo_surface_t *surface);

void DrawContour2D::Messaggio(bool ifMess)
{
   stringstream mess;
   if(Contour == nullptr)
   {
       mess.str(" ");
       return;
   }
   if(ifMess)
   {	   
      if(Contour->sezione0 == Contour->sezioneN)
            mess << "Section " << Contour->sezione0+1 << " of "
                 <<  ngrid[Contour->Layer-1] << " along "
                 <<  layers[Contour->Layer-1] << ";   "
                 << layers[Contour->Layer+2] << " = "
                 << (float)Contour->sezione0/(float)ngrid[Contour->Layer-1];
      else
            mess << "Sections " << Contour->sezione0+1 << " to "
                 << Contour->sezioneN+1
                 << " of " << ngrid[Contour->Layer-1] << " along "
                 << layers[Contour->Layer-1] << ";   "
                 << (float)Contour->sezione0/(float)ngrid[Contour->Layer-1]
                 << " < " << layers[Contour->Layer+2] << " < "
                 << (float)Contour->sezioneN/(float)ngrid[Contour->Layer-1];
   }
   else
       mess.str(" ");
   if(m_gl)
      m_gl->PrintMess(mess.str().c_str());
}
   
DrawContour2D::DrawContour2D():
	Contour(nullptr),
	m_gl(nullptr)
{
  if(GLWINDOW)
     m_gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW),
                       "glwindow");
  m_width = m_gl->Width;
  m_height = m_gl->Height;
  if(Contour)
     delete Contour;
  Contour = new myContour;
  getinfocontour_(ngrid, Contour);
  Messaggio(true);
}

void DrawContour2D::LoadTextureFile(const char *szFilename, MyTex *pTexture)
{
    GdkPixbuf *pixbuf = create_pixbuf  (szFilename);
    if(pixbuf == NULL)
       return;
    int  width = gdk_pixbuf_get_width (pixbuf);
    int  height = gdk_pixbuf_get_height (pixbuf);
    int  n_channels = gdk_pixbuf_get_n_channels (pixbuf);
    int  rowstride = gdk_pixbuf_get_rowstride (pixbuf);

    pTexture->ulDimensionX = width;
    pTexture->ulDimensionY = height;
    pTexture->n_channels = n_channels;
    if(n_channels == 4)
       pTexture->format = GL_RGBA;
    else
       pTexture->format = GL_RGB;
    pTexture->rowstride = rowstride;
    pTexture->pcData = gdk_pixbuf_get_pixels (pixbuf);
}

void DrawContour2D::LoadDrawTexture(const char *textfile)
{
  MyTex *TextureImage = g_new0(MyTex, 1);
  LoadTextureFile(textfile, TextureImage);
  glGenTextures(1, &(m_Texture));
  glBindTexture(GL_TEXTURE_2D, m_Texture);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, TextureImage->n_channels,
	TextureImage->ulDimensionX, TextureImage->ulDimensionY, 0,
	TextureImage->format, GL_UNSIGNED_BYTE, TextureImage->pcData);
  if (TextureImage)                      // If Texture Exists
  {
     if (TextureImage->pcData)            // If Texture Image Exists
         g_free(TextureImage->pcData);        // Free The Texture Image Memory
     g_free(TextureImage);                  // Free The Image Structure
  }
}

void DrawContour2D::DrawTexture()
{
    if(m_Texture == 0)
       return;
    float X1 = -1.f;
    float Y1 = -1.f;
    float X2 =  1.f;
    float Y2 =  1.f;
    float X3=1.f, Y3=1.f;
    if(m_gl)
    {
       if(m_gl->aspectRatio <= 1)
       {
	  X1 *= m_gl->aspectRatio;
	  X2 *= m_gl->aspectRatio;
       }		
       else
       {
	  Y1 /= m_gl->aspectRatio;
	  Y2 /= m_gl->aspectRatio;
       }		
    }

    float scal = 1.f/min(X2, Y2);
    X1 *= scal;
    X2 *= scal;
    Y1 *= scal;
    Y2 *= scal;
    glDisable (GL_LIGHTING);
    glDisable(GL_CLIP_PLANE0);
    glDisable(GL_CLIP_PLANE1);
    glDisable(GL_FOG);

    glDisable(GL_DEPTH_TEST);
    glEnable( GL_BLEND );
    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
    glBindTexture(GL_TEXTURE_2D, m_Texture);
    glEnable (GL_TEXTURE_2D);

    glBegin(GL_POLYGON);                                          // Front Face 
     glTexCoord2f(0.0f, 0.0f); glVertex3f(X1, Y2,  1.0f);  // Bottom left Of The Texture and Quad
     glTexCoord2f(X3, 0.0f); glVertex3f(X2, Y2,  1.0f);  // Bottom right Of The Texture and Quad
     glTexCoord2f(X3, Y3); glVertex3f(X2, Y1,  1.0f);  // Top right Of The Texture and Quad
     glTexCoord2f(0.0f, Y3); glVertex3f(X1, Y1,  1.0f);  // Top left Of The Texture and Quad
    glEnd();
    
    glDisable (GL_TEXTURE_2D);
}

void DrawContour2D::StartContour()
{
    m_image = cairo_image_surface_create(CAIRO_FORMAT_RGB24,
                        m_width, m_height);
    m_cr = cairo_create (m_image);
    NewPage();
}

void DrawContour2D::NewPage()
{
    cairo_set_source_rgb(m_cr, 1, 1, 1);
    cairo_rectangle(m_cr, 0, 0, m_width, m_height);
    cairo_fill(m_cr);
}

void DrawContour2D::DrawLine(int x[], int y[], float Color[], int Type, int Lw)
{
    double dashes[] = {2, 2};
    int ndash  = sizeof (dashes)/sizeof(dashes[0]);
    double offset = -2.0;
    cairo_set_source_rgb(m_cr, Color[0], Color[1], Color[2]);
    cairo_set_line_width(m_cr, Lw);
    if(Type == 2)
       cairo_set_dash (m_cr, dashes, ndash, offset);
    else
       cairo_set_dash (m_cr, dashes, 0, offset);
    cairo_move_to(m_cr, x[0], y[0]);
    cairo_line_to(m_cr, x[1], y[1]);
    cairo_stroke(m_cr);
}

void DrawContour2D::DrawLabel(int x, int y, char *str, float *Color, int Lw)
{
    cairo_move_to(m_cr, x, y);
    cairo_set_source_rgb(m_cr, Color[0], Color[1], Color[2]);
    cairo_set_font_size(m_cr, 20);
    cairo_show_text(m_cr, str);
}

void DrawContour2D::EndContour()
{
    GError *error=NULL;
    string filname = getFilePng(m_gl);
#ifdef IN_GTK3
    GdkPixbuf *pixbuf = gdk_pixbuf_get_from_surface (m_image, 0, 0, m_width, 
		    m_height);
#else
    GdkPixbuf *pixbuf = fromCairoToPixbuf (m_image);
#endif
    GdkPixbuf *dest = gdk_pixbuf_add_alpha(pixbuf, FALSE, 255, 255, 255);
    gdk_pixbuf_save (dest, filname.c_str(), "png", &error, NULL);
    g_object_unref(pixbuf);
    g_object_unref(dest);	
    cairo_destroy(m_cr);
    cairo_surface_destroy(m_image);
    LoadDrawTexture(filname.c_str());
    //DrawTexture();
}

void DrawContour2D::ChangeSection(int quanto, GtkWidget *window)
{
     if (quanto < 5)
     {
       Contour->sezione0 += quanto;
       Contour->sezioneN += quanto;
       if(Contour->sezione0 > (ngrid[Contour->Layer-1]-1))
          Contour->sezione0 = 0; 
       else if(Contour->sezione0 < 0)
          Contour->sezione0 = ngrid[Contour->Layer-1]-1;
       if(Contour->sezioneN > (ngrid[Contour->Layer-1]-1))
          Contour->sezioneN = 0; 
       else if(Contour->sezioneN < 0)
          Contour->sezioneN = ngrid[Contour->Layer-1]-1;
     }
     else
     {
       Contour->sezione0 = 0;
       Contour->sezioneN = Contour->sezione0;	     
     }
     bool mess = false;
     if (quanto < 5)
     {
        GtkWidget *ht = get_widget(window, "ctoolbar");
        gtk_widget_set_sensitive(ht, FALSE);
	//preparacontour_();
        leggisezioni_();
        gtk_widget_set_sensitive(ht, TRUE);
	mess = true;
     }
     Messaggio(mess);
     putinfocontour_(Contour);
     plotcontourjav_();
}

void DrawContour2D::ReDraw()
{
     if(m_gl)
         m_gl->Redraw();
}

GdkColor *get_colorgdk(int new_color);

static DrawContour2D *contour=nullptr;
static ContDlg1 *cont_dlg1=nullptr;
static ContDlg2 *cont_dlg2=nullptr;
static ContDlg3 *cont_dlg3=nullptr;


void initcontourJav(int *w, int *h)
{
     initcontourjav_(w, h);
}

extern "C" void  startcontour2d_()
{
    if(contour)
       delete contour;
    contour = new DrawContour2D();
    contour->StartContour();
}

extern "C" void drawlinec2d_(int x[], int y[], int *icol, int *tipo, int *lw)
{
    if(contour == nullptr)
       return;
    float *color = getfColor(*icol);
    contour->DrawLine(x, y, color, *tipo, *lw);
}

extern "C" void drawlabeljav_(int *x, int *y, char *str, int *icol, int *lw)
{
    if(contour == nullptr)
       return;
    float *color = getfColor(*icol);
    contour->DrawLabel(*x, *y, str, color, *lw);
}

extern "C" void  stopcontour2d_()
{
    if(contour == nullptr)
       return;
    contour->EndContour();
    contour->ReDraw();
}

void draw_2d_contour()
{
    if(contour == nullptr)
       return;
    contour->DrawTexture();
}

void ChangeSectionContour(int quanto, GtkWidget *window)
{
    if(contour == nullptr)
       return;
    contour->ChangeSection(quanto, window);
}

void ViewOptions(gpointer item, gpointer data)
{
    int action = GPOINTER_TO_INT(data);

    viewforjav_(&action);
}

void doopenContourDlg1()
{
   if(cont_dlg1 == nullptr)
   {
      cont_dlg1 = new ContDlg1();
      cont_dlg1->SettaContour(contour->getContour());
   }
   cont_dlg1->ShowDlg(GLWINDOW); 
}

void doopenContourDlg2()
{
   if(cont_dlg2 == nullptr)
   {
      cont_dlg2 = new ContDlg2();
      cont_dlg2->SettaContour(contour->getContour());
   }
   cont_dlg2->ShowDlg(GLWINDOW); 
}

void doopenContourDlg3()
{
   if(cont_dlg3 == nullptr)
   {
      cont_dlg3 = new ContDlg3();
      cont_dlg3->SettaContour(contour->getContour());
      cont_dlg3->SettaGrid(contour->getGrid());
   }
   cont_dlg3->ShowDlg(GLWINDOW); 
}

void SetCell(int active)
{
    int ifcell = active;
    setcelldraw_(&ifcell);
    plotcontourjav_();
}

void SetGrid(int active)
{
    int ifgrid = active;
    setgriddraw_(&ifgrid);
    plotcontourjav_();
}

void AggiornaStatusBar()
{
    if(contour == nullptr)
       return;
    contour->Messaggio(true);
}
