#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include "utils.h"
#include "sir_application.h"
#include <csymlib.h>
#include <cmtzlib.h>
#include <sstream>
#include <fstream>
#include <iomanip>

char *get_spg(char *spgstring, int ifSilent)
{
   char *spgstr = g_new0(char, 25);
   CopiaStringan(spgstr, 25, g_strstrip(spgstring), 24);
   spgstr[24] = '\0';
   if(!isdigit((int) spgstr[0]))
   { 
//      if(ifSilent)
//         return spgstr;
      CSym::CCP4SPG *spg = CSym::ccp4spg_load_by_spgname(spgstr);
      if(spg != NULL)
      {
         CSym::ccp4spg_free(&spg);
         return spgstr;
      }
      else
      {
         int newsi = (int) strlen(spgstr) + 4;
         char *newspgstr = g_new0(char, newsi);
         CopiaStringa(newspgstr, newsi, spgstr);
         AppendiStringa(newspgstr, newsi, " :2");
         CSym::CCP4SPG *spg = CSym::ccp4spg_load_by_spgname(newspgstr);
         if(spg != NULL)
         {
            CSym::ccp4spg_free(&spg);
            g_free(spgstr);
            return newspgstr;
         }
      }
      if(!ifSilent)
         gtkext::dlg_errmsg(NULL, "Error in SpaceGroup Symbol %s", spgstr);
      memset(spgstr, ' ', 24);                //120612
   }
   else
   {
      int spgr = atoi(spgstring);
      char *spgrname = g_new0(char, MaxLenFilename);
      CSym::CCP4SPG *spg = CSym::ccp4spg_load_by_standard_num(spgr);
      if(spg != NULL)
      {
         ScriviStringa(spgrname, MaxLenFilename, "%s", spg->symbol_xHM);
         CSym::ccp4spg_free(&spg);
         return spgrname;
      }
      if(!ifSilent)
         gtkext::dlg_errmsg(NULL, "Error in SpaceGroup Number %d", spgr);
    }
//    memset(spgstr, ' ', 24);
    return spgstr;
}

int GetSpgNum(char *spacegroupname, float r[], int *nsymop)
{
if(strlen(spacegroupname) == 0)
   return -1;
CSym::CCP4SPG *spg = CSym::ccp4spg_load_by_spgname(spacegroupname);
if(spg == NULL)
   return -1;
int spgr = spg->spg_num;
if(nsymop != NULL)
   *nsymop = spg->nsymop;
if(spg->nsymop_prim >=2)
{
   for(int i=0; i<3; i++)
       r[i] = spg->symop[1].rot[i][i];
}
else
{
   for(int i=0; i<3; i++)
       r[i] = 0;
}
CSym::ccp4spg_free(&spg);
return spgr;
}

void myshow_mtz(const std::string &fn, GtkWindow *window)
{
using namespace std;
    CMtz::MTZ *mtzin;
    CMtz::MTZSET *mtzset;

    char titl[1024];
    int i, j, k, nRefl;
    float resMin, resMax, aresMin, aresMax;
    float cell[6], wave;
    double coefhkl[6];
    int nsympx, nspgrx, nsets, ncrys;
    char ltypex[256], spgrnx[256], pgnamx[256];
    char labels[100][31], types[100][3];
    char crysname[64], dsname[64], prjname[64];
    int sets[100], ind[3];
    int ncols, id;
    std::ofstream os;
    std::stringstream os1, os2;
    size_t sz = fn.length() + 1;
    gchar *mtz_out = g_new0(char, sz);

    if( !g_file_test(toUTF8(fn.c_str()), G_FILE_TEST_EXISTS) )
    {
        gtkext::dlg_errmsg(NULL, "File does not exist: %s\n", fn.c_str());
        return;
    }
    const char *mtzname = fromUTF8(fn).c_str();
    mtzin = CMtz::MtzGet(mtzname, 0);
    if( mtzin==0 )
    {
        gtkext::dlg_errmsg(NULL, "Error in open file: %s\n", fn.c_str());
        return;
    }

    CopiaStringa(mtz_out, sz, (fn.substr(0, fn.length()-3)).c_str());
    AppendiStringa(mtz_out, sz, "txt");
    os.open(mtz_out);
    if( !os || !os.is_open() )
    {
        gtkext::dlg_errmsg(NULL, "failure file open: %s\n", fn.c_str());
        return;
    }

    CMtz::ccp4_lrtitl(mtzin, titl);
    os << "Title: " << titl << "\n\n";

    nsets = MtzNset(mtzin);
    ncrys = MtzNxtal(mtzin); 
    os << "Number of datasets: " << nsets << "\n\n";
    os << "Dataset ID, project/crystal/dataset names:\n\n";
    for(j=0; j<ncrys; j++)
    {
        nsets = MtzNsetsInXtal(MtzIxtal(mtzin, j));
        for(i=0; i<nsets; i++)
        {
         mtzset = MtzIsetInXtal(MtzIxtal(mtzin, j), i);
         ccp4_lridx(mtzin, mtzset, crysname, dsname, prjname, &id, cell, &wave); 
         os << id << "   " << prjname << ", ";
         os << crysname << ", ";
         os << dsname << "\n";
         os << "Wavelength: " << wave << "\n\n";
        }
    }
    for (j = 0; j < ncrys; ++j)
    {
         if (mtzin->xtal[j]->cell[0] > 0.001) 
         {
             CMtz::MtzHklcoeffs(mtzin->xtal[j]->cell, coefhkl);
             break;
         }
    }


    ncols = CMtz::MtzNcol(mtzin);
    CMtz::MtzListColumn(mtzin, labels, types, sets);
    CMtz::ccp4_lrsymi(mtzin, &nsympx, ltypex, &nspgrx, 
       spgrnx, pgnamx);
    CSym::CCP4SPG *spg = CSym::ccp4spg_load_by_spgname(spgrnx);
    CSym::ccp4spg_free(&spg);
    CMtz::ccp4_lrcell(MtzIxtal(mtzin, 0), cell);
    nRefl = CMtz::MtzNref(mtzin);
    CMtz::MtzResLimits(mtzin, &resMin, &resMax);
    try
    {
       aresMin = (float) (1.0 / sqrt((double)resMin));
       aresMax = (float) (1.0 / sqrt((double)resMax));
    }
    catch(...)
    {
    }
    CMtz::MtzNref(mtzin);

    os << setiosflags( ios::fixed );
    os << setiosflags( ios::right );
    os << setprecision( 2 );
    os << setw(20);
    os << "Number of Columns = " << ncols << "\n\n";
    os << resetiosflags( ios::right );
    os << setiosflags( ios::internal );
    os << "                          OVERALL FILE STATISTICS\n";
    os << "                          _______________________\n\n";
    os << resetiosflags( ios::internal );
    os << "Col# " << " Type" << setw(8) << " Label" << setw(18) << "Min" << setw(12) ;
    os <<"Max" << setw(8) << "    Missing   % complete    Mean      Mean abs.";
    os << "  Res.Low   Res.High \n\n";
    os << resetiosflags( ios::internal );
    os << setiosflags( ios::right );


    float *refl = g_new0(float, nRefl*ncols);
    float *resol = g_new0(float, nRefl);
    CMtz::MtzRrefl (mtzin->filein, ncols*nRefl, refl);
    float **refl_data = g_new0(float *, 10);
    for(i=0; i<10; i++)
        refl_data[i] = g_new0(float, ncols);
    int *missing  = g_new0(int, ncols);
    float *somma = g_new0(float, ncols);
    float *sommaa = g_new0(float, ncols);
    float *mini = g_new0(float, ncols);
    float *maxi = g_new0(float, ncols);
    float *reslow = g_new0(float, ncols);
    float *reshig = g_new0(float, ncols);
    for(i=0; i<ncols; i++)
    {
        mini[i] = 1.e6;
        reslow[i] = -1.e6;
        maxi[i] = -1.e6;
        reshig[i] = 1.e6;
    }
    k = 0;
    for(j=0; j<nRefl; j++)
    {
        ind[0] = (int) refl[k];
        ind[1] = (int) refl[k + 1];
        ind[2] = (int) refl[k + 2];
        resol[j] = CMtz::MtzInd2reso(ind, coefhkl);
        resol[j] = (float) (1.0 / sqrt((double)resol[j]));
        if(resol[j] < aresMax) resol[j] = aresMax;
        if(resol[j] > aresMin) resol[j] = aresMin;
        for(i=0; i<ncols; i++)
        {
            char *num = g_strdup_printf("%f", refl[k]);
            if(strstr(g_ascii_strup(num, (gsize) strlen(num)), "NAN") )
                missing[i]++;
            else
            {
                somma[i] += refl[k];
                sommaa[i] += MRabs(refl[k]);
                if(mini[i] > refl[k])
                   mini[i] = refl[k];
                if(maxi[i] < refl[k])
                   maxi[i] = refl[k];
                if(resol[j] < reshig[i])
                   reshig[i] = resol[j];
                if(resol[j] > reslow[i])
                   reslow[i] = resol[j];
            }
            if(j < 10)
            {
                char *num = g_strdup_printf("%f", refl[k]);
                if(strstr(g_ascii_strup(num, (gsize) strlen(num)), "NAN") )
                {
                   if(i < 3)
                      refl_data[j][i] = -999;
                   else
                      refl_data[j][i] = -9999.f;
                }
                else
                   refl_data[j][i] = refl[k];
            }
            k++;
        }
      }
      for(i=0; i<ncols; i++)
      {
        float completo = (float)(nRefl-missing[i]) / (float)nRefl * 100.f;
        float media = 0, mediaa = 0;
        if((nRefl-missing[i]) != 0)
        {
           media = somma[i] / (float) (nRefl-missing[i]);
           mediaa = sommaa[i] / (float) (nRefl-missing[i]);
        }
        os << setw(3) <<  i+1  
           << "  " << setw(3) << types[i] << "     "
           << resetiosflags (ios::right) << setiosflags( ios::left ) << setw(12) << labels[i]
           << resetiosflags (ios::left) << setiosflags( ios::right ) << setw(12) ;
        if(types[i][0] == 'H')
           os << iround(mini[i]) <<  setw(12) << iround(maxi[i]);
        else
           os << mini[i] <<  setw(12) << maxi[i];
        os << setw(8) << missing[i] << setw(12) << completo 
           <<  setw(12) << media << setw(12) << mediaa 
           <<  setw(10) << reslow[i] << setw(10) << reshig[i] << " \n";
    }
    g_free(refl);
    g_free(mini);
    g_free(maxi);
    g_free(reslow);
    g_free(reshig);
    g_free(somma);
    g_free(sommaa);
    g_free(missing);

    os << "\n\n";
    os << setprecision( 4 );
    os << "Cell dimensions:\n" << cell[0] << " " << cell [1]  << " " << cell [2]
     << " " << cell [3] << " " << cell [4] << " " << cell [5] << "\n\n";
    os << "Number of Reflections = " << nRefl << "\n\n";
    os << "Resolution Range:\n" << resMin << " - " << resMax 
       << "(" << aresMax << " - " << aresMin << " A)\n\n";
    os << "Space group: " << spgrnx << "(number " << nspgrx << ")\n\n";

    os << setiosflags( std::ios::fixed );
    os << setiosflags( std::ios::right );
    os << std::setprecision( 3 );

    os << "List of first 10 reflections\n\n";
    for(j=0; j<=10; j++)
    {
      for(i=0; i<ncols; i++)
      {
        if(j == 0)
        {
           os << resetiosflags( ios::right );
           os << setiosflags( ios::internal );
           if(i < 3)
              os << std::setw( 5 );
           else
              os << std::setw( 12 );
           os << resetiosflags( ios::right ) << setiosflags( ios::left ) << g_strstrip(labels[i]);
        }
        else
        {
           if(i < 3)
           {
              os << std::setw( 5 ) << resetiosflags( ios::left ) << setiosflags( ios::right );
              os <<  iround(refl_data[j-1][i]);
           }
           else
           {
              os << std::setw( 12 ) << resetiosflags( ios::left ) << setiosflags( ios::right );
              os << refl_data[j-1][i];
           }
        }
      }
      if(j == 0)
           os << "\n";
      os << "\n";
    } 


    for(i=0; i<10; i++)
        g_free(refl_data[i]);
    g_free(refl_data);

    os.close();
    MtzFree(mtzin);
    gtkext::text_window(window, mtz_out, "*.txt", 
         gtkext::IFREADONLY, fn.c_str(), 850, 500);
  }

int GetMtzCols(std::string fn, std::string &MtzCols)
 {
    char labels[100][31], types[100][3];
    int sets[100];
    int ncols;
    CMtz::MTZ *mtzin;

    const char *mtzname = fromUTF8(fn).c_str();
    mtzin = CMtz::MtzGet(mtzname, 0);
    if(!mtzin)
       return -1;
    ncols = CMtz::MtzNcol(mtzin);
    CMtz::MtzListColumn(mtzin, labels, types, sets);

    for( int i=0; i<ncols; i++ )
    {
      MtzCols.append( (CMtz::MtzColLookup(mtzin, labels[i])->label) );
      MtzCols.append( " " );
    }

    return 0;
  }

static void fill_combo_box (GtkWidget *combo, GList *strings)
{
    int nlabels = g_list_length(strings);
    if (nlabels == 0)
        return;
#ifndef GTK_COMBO_BOX
    gtk_combo_set_popdown_strings(GTK_COMBO(combo), strings);
#else
    int i;
    for(i=0; i<nlabels; i++)
        combo_remove_text (combo);
    for(i=0; i<nlabels; i++)
        combo_insert_text (combo, i, (char *)g_list_nth_data(strings, i));
#endif
}

static void combo_entry_activated(GtkWidget *entry, gpointer data)
{
   int hnd = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(entry), "hnd"));
   int ifSilent = GPOINTER_TO_INT(data);
   char *string = g_new0(char, 25);
   CopiaStringan(string, 25, gtk_entry_get_text(GTK_ENTRY(entry)), 24);
   string[24] = '\0';
   if(str_white(string))
      return;
   g_signal_handler_block(G_OBJECT(entry), hnd);
   gtk_entry_set_text(GTK_ENTRY(entry), get_spg(string, ifSilent));
   g_signal_handler_unblock(G_OBJECT(entry), hnd);
}

gboolean on_entry_focus_out_event  (GtkWidget  *, GdkEventFocus *,
                                    GtkWidget *widget)
{
  g_signal_emit_by_name  (G_OBJECT(widget), "activate", widget,
            GINT_TO_POINTER(0));
  return FALSE;
}

GtkWidget *spgr_widget(const char *filespg, int ifSilent)
{
   GtkWidget *box, *entry;
   GList *glist = NULL;
   char   *spg;
   char   filspg[MAX_LEN];
   FILE   *fp;
   int i;

   fp = fOpen(filespg, "r");
   if(fp != NULL)
   {
      spg = g_new0(char, 17);
      for(i=0; i<26; i++)
      {
          if(fgets(filspg, MAX_LEN, fp) != NULL)
             continue;
      }
      i = 0;
      for(;;)
      {
         if(fgets(filspg, MAX_LEN, fp) != NULL)
         {
            if(filspg[0] != '$' && strlen(filspg))
            {
                (void) CopiaStringan(spg, 17, filspg, 12);
                *(spg + 12) = '\0';
                glist = g_list_append(glist, DuplicaStringa(g_strstrip(spg)));
            }
            else
                break;
         }
         else
            break;
      }
      (void) fclose(fp);
   }

#ifndef GTK_COMBO_BOX
   box = gtk_combo_new();
   entry = GTK_COMBO(box)->entry;
   gtk_combo_disable_activate(GTK_COMBO(box));
   g_object_set_data (G_OBJECT (GTK_COMBO(box)->list), "entry", entry);
#else
   box = combo_new();
   entry = gtk_bin_get_child(GTK_BIN(box));
#endif
   g_object_set_data (G_OBJECT(box), "entry", entry);
   fill_combo_box(box, glist);
   gtk_entry_set_text(GTK_ENTRY(entry), " ");
   set_width_chars (GTK_ENTRY (entry), 16);
#ifndef GTK_TOOLTIP
   GtkTooltips *myToolTips = gtk_tooltips_new();
   g_object_set_data (G_OBJECT(box), "tt", myToolTips);
#endif

   int hnd = g_signal_connect(G_OBJECT(entry), "activate", G_CALLBACK(combo_entry_activated), 
       GINT_TO_POINTER(ifSilent));
   g_object_set_data (G_OBJECT(entry), "hnd", GINT_TO_POINTER(hnd));
   g_signal_connect_after (G_OBJECT(entry), "focus_out_event",
                   G_CALLBACK (on_entry_focus_out_event), entry);

#ifdef GTK_ENTRY_COMPLETION
  GtkTreeIter iter;
  GtkListStore *completion_model;
  GtkEntryCompletion *completion = gtk_entry_completion_new ();
  gtk_entry_set_completion (GTK_ENTRY (entry), completion);
  completion_model = gtk_list_store_new(1, G_TYPE_STRING);
  for(i=0; i < (int)g_list_length(glist);i++)
  {
      gtk_list_store_append(completion_model, &iter);
      gtk_list_store_set(completion_model, &iter, 0, g_list_nth_data(glist, i), -1);
  }
  gtk_entry_completion_set_model (completion, GTK_TREE_MODEL(completion_model));
  gtk_entry_completion_set_text_column (completion, 0);
#endif

   g_list_free(glist);
   gtk_widget_show_all(box);
   gtk_widget_set_size_request(box, 110, -1);
   set_tip (entry, "Select the Space Group Symbol from the list ");

   return box;
}

std::string TraslToChar(float trval)
{
   int n = (int) (trval * 100);
   std::string out;
   switch(n)
   {
   case 50:
        out = "1/2";
        break;
   case 33:
        out = "1/3";
        break;
   case 66:
   case 67:
        out = "2/3";
        break;
   case 25:
        out = "1/4";
        break;
   case 75:
        out = "3/4";
        break;
   case 16:
   case 17:
        out = "1/6";
        break;
   case 82:
   case 83:
        out = "5/6";
        break;
   default:
        out = "   ";
        break;
   }
   return out;
}

void BuildSymmOps(CSym::CCP4SPG *spg, std::ostream &strm)
{
   const char *xyzstr = "xyz";
   float EPS = 0.001f;
   strm << "Symmetry operations\n";
   for(int i=0; i<spg->nsymop; i++)
   {
       strm << "     " << i+1 << ") ";
       float trv[3];
       for(int j1=0; j1<3; j1++)
       {
           float tr = spg->symop[i].trn[j1];
           if(tr < 0)
              tr += 1.0f;
           trv[j1] = tr;
       }
       for(int j1=0; j1<3; j1++)
       {
           bool sign = false;
           if(trv[j1] > EPS)
           {
              strm << TraslToChar(trv[j1]);
              sign = true;
           }
           for(int j2=0; j2<3; j2++)
           {
               int ii = (int) spg->symop[i].rot[j1][j2];
               if(ii)
               {
                  if(ii < 0)
                     strm << "-";
                  else if(sign)
                     strm << "+";
                  strm << xyzstr[j2];
                  sign = true;
               } 
           }
           if(j1 < 2)
           {
              strm << ",";
              sign = false;
           }
       }
       strm << "\n";
   }
}

void Read_PDB(const std::string &fn, int &nres0, int &nres1, int &numeroResidui)
  {
    std::ifstream f(fromUTF8(fn).c_str());
    std::vector<int> vresi;
    std::vector<int>::iterator it;
    bool endchain = false;
    nres0 = 1000;
    nres1 = 0;
    numeroResidui = 0;

    if( !f )
    {
      gtkext::dlg_errmsg(NULL, "failed to open file:  %s", fn.c_str());
      return;
    }
    std::string line;

    while( !f.eof() )
    {
      std::getline(f, line);
      endchain = false;
      if( g_ascii_strncasecmp(line.c_str(), "ATOM", 4)==0 )
      {
         int nn = IsAmino(line.substr(17, 3).c_str());
         if(nn > 0)
         {
            int num = atoi(line.substr(22, 4).c_str());
            vresi.push_back(num);
         }
      }
      if( g_ascii_strncasecmp(line.c_str(), "TER", 3)==0 )
      {
          endchain = true;
          if(vresi.size() > 0)
          {
             sort(vresi.begin(), vresi.end());
             it = unique(vresi.begin(), vresi.end());
             vresi.resize( it - vresi.begin() );
             nres0 = MRmin(vresi.at(0), nres0);
             nres1 = MRmax(vresi.at(vresi.size()-1), nres1);
             numeroResidui += (int) vresi.size();
             vresi.clear();
          }
      }
    }
    f.close();
    if(!endchain)
    {
       if(vresi.size() > 0)
       {
          sort(vresi.begin(), vresi.end());
          it = unique(vresi.begin(), vresi.end());
          vresi.resize( it - vresi.begin() );
          nres0 = MRmin(vresi.at(0), nres0);
          nres1 = MRmax(vresi.at(vresi.size()-1), nres1);
          numeroResidui += (int) vresi.size();
       }
    }
}
