!#define FFTW

MODULE fouriermap

implicit none

integer, parameter             :: FOUR_FO=1, FOUR_2FO_FC=2, FOUR_FO_FC=3, FOUR_FC=4
character(len=8), dimension(4) :: four_name = [ 'Fo      ', '2Fo - Fc', 'Fo - Fc ', 'Fc      ' ] 

real :: maxfmap

type map_info_type 
   logical :: active   ! map visualization active?
   integer :: ftype    ! map type? 
end type map_info_type

type(map_info_type) :: mapinfo

CONTAINS


   subroutine labelx(atmold,cell,spg,nPeaksToFind,ier)
   USE General, only: lo, StructureName, celem
   USE Symmetry, only: ddmin
   USE Molcom, only: It
   USE sinterface
   USE ilmviewmod
   USE variables
   USE atom_type_util
   USE fragmentmod
   USE connect_mod
   USE unit_cell
   USE PhaseModule, only:biso
   USE strutil
   USE cgeom
   USE spginfom
!
   implicit none
   type(atom_type), dimension(:), allocatable, intent(in) :: atmold
   type(cell_type), intent(in)                            :: cell
   type(spaceg_type), intent(in)                          :: spg
   integer, intent(in)                                    :: nPeaksToFind
   type(fragment_type), dimension(:), allocatable         :: fragm
   integer                                                :: nfrag
   real, dimension(3)                                     :: xn,xo,xeq,s
   real, dimension(2)                                     :: vet
   real, dimension(:), allocatable                        :: tempdist
   real, allocatable, dimension(:)                        :: distminv
   integer, allocatable, dimension(:)                     :: kminv,selected,npunt
   integer                                                :: i, j, k, ifi, kt, jpunt, ifin
   integer                                                :: Naccept
   integer                                                :: ier
   integer                                                :: iExitC,MultiSel,NumCan
   real                                                   :: d, distm
   integer                                                :: iat, nnzo
   integer                                                :: nat_old, nat_new
   type(atom_type), dimension(:), allocatable             :: atmnew
   logical                                                :: kpr = .true.
   integer                                                :: new_peaks
   real                                                   :: coef
   character(len=:), allocatable                          :: msg
   integer                                                :: iAction
!corr   type(cell_type) :: cellt
!
   nat_old = numatoms(atmold)
   call xyzs_to_atom(atmnew)
   nat_new = numatoms(atmnew)
   call atom_string(atmnew,celem)  ! peaks will be as Q1, Q2, ...
   atmnew(:)%biso=biso
   if (kpr) then
       call print_atoms(atmold,title='Old atoms list')
       call print_atoms(atmnew,title='Fourier peaks',height=.true.)
   endif
   allocate(tempdist(nat_new))
   allocate(npunt(nat_new),source=0)
   vet(:) = (/1.0,-1.0/)
!corr   ifin=icent+1
!corr   qq(:) = 0.0
   if (spg%symcent == 1) then
       ifin = 2
   else
       ifin = 1
   endif
!corr   cellt = set_cell_type(cell)
   do i=1,nat_old
      !nnzo=iabs(atmold(i)%nz)/nzConst
      nnzo=atmold(i)%kscatt()
      xo(:) = atmold(i)%xc(:)
      distm=9999
      jpunt=0
      do j=1,nat_new
!corr         if (qq(j).lt.0.5) then
         if (npunt(j) == 0) then
             xn(:) = atmnew(j)%xc(:)
             do kt=1,spg%ncoper
                do ifi=1,ifin
                   coef=vet(ifi)
                   do k=1,spg%nsym
                      call xequi(xo,k,coef,xeq,kt)
!                      call xdist(xn,xeq,d,s)
                      call xdisteqs(xn,xeq,cell%get_g(),d,s)
                      if (d.lt.ddmin) then
                          distm=d
                          !jpunt=j
                          !qq(jpunt) = i
                          !tempdist(jpunt) = d
                          npunt(j) = i
                          tempdist(j) = d
                      endif
                   enddo
                enddo
             enddo
         endif
      enddo
   enddo
!
   if (kpr) then
       do j=1,nat_new
!corr          i=nint(qq(j))
!test for real              write(0,*)'ATM=',j,atmnew(j)%inte
          i=npunt(j)
          if (i > 0) then
              write(lo,'(2x,a,f8.2,4f10.3,10x,a8,i4,4f10.3,1x,f10.3)')             &
              atmnew(j)%lab,atmnew(j)%inte,atmnew(j)%xc,atmnew(j)%biso,    &
              atmold(i)%lab,i,atmold(i)%xc(:),atmold(i)%biso,tempdist(j)
          else
              !corr write(lo,'(2x,a,i5,4f10.3)') atmnew(j)%lab,nint(atmnew(j)%inte),atmnew(j)%xc,atmnew(j)%biso
              write(lo,'(2x,a,f8.2,4f10.3)') atmnew(j)%lab,atmnew(j)%inte,atmnew(j)%xc,atmnew(j)%biso
          endif
       enddo
   endif
   new_peaks = 0
!corr   nequal = 0
!corr   sumold = 0.0
!corr   sumnew = 0.0
!corr!
!corr!  Scale intensities
!corr   scalatom = 1.0
!corr   if (icalfo /= FOUR_FO_FC) then
!corr       do j=1,nat_new
!corr          i=npunt(j)
!corr          if (i > 0) then
!corr              nequal = nequal + 1
!corr              sumold = sumold + atmold(i)%inte
!corr              sumnew = sumnew + atmnew(j)%inte
!corr          endif
!corr!          new_peaks = new_peaks + 1
!corr       enddo
!corr       if (nequal > 0 .and. sumnew > 0) scalatom = sumold/sumnew
!corr   endif
!
   do j=1,nat_new
!
!     accept the new peak only if has no counterpart
!     in the atom list
!
      i=npunt(j)
      if (i == 0) then
          new_peaks = new_peaks + 1
          !atmnew(new_peaks)%xc = atmnew(j)%xc
          atmnew(new_peaks) = atmnew(j)
          atmnew(new_peaks)%biso = biso
      endif
   enddo
!corr!
!corr! Riscala intensita
!corr!
!corr   scalatom = 1.0
!corr  !!!!FIXME-non funziona                     atmnew(:)%inte = atmnew(:)%inte / umapel
!corr!corr   if (nequal.gt.0.and.icalfo.ne.3) then
!corr   if (nequal > 0 .and. icalfo /= FOUR_FO_FC) then
!corr       if (sumnew.gt.0.0) scalatom = sumold/sumnew
!corr   endif
!
   if (new_peaks > nPeaksToFind) new_peaks = nPeaksToFind
!
   nat_new = new_peaks + nat_old
   call resize_atoms(atmnew,nat_new)
   if (new_peaks > 0) then
       write(lo,"(10x,i5, '  new peak(s) found')") new_peaks
!corr  265     format(  10x,i5, '  new peak(s) found')
       call calcola_occ(atmnew(:new_peaks),spg,cell)
!
!         move atoms at the end of arrays
!         to make room for old atoms
       atmnew(nat_old+1:) = atmnew(:new_peaks)
   endif
!
!  copy old atoms at the beginning of arrays
   do i=1,nat_old
      atmnew(i) = atmold(i)
   enddo
!corr   atmnew(:nat_old) = atmold(:)
!
   if (new_peaks > 0) then
       call copy_atoms(atm,atmnew)
!
!      Frammenta il modello
       call get_fragments(atm,cell,lconn,nfrag,fragm)
!
!      Raggruppa i frammenti 
       call regroup_fragment(fragm,atm,cell,spg)
!corr!
!corr!      Disegna il modello e i nuovi atomi come ghost
!corr       if (.not.jav_opened())call ilmview_open(StructureName)
!corr       call jav_set_modify(0)
!corr       call ilmview_draw(atm,lconn,celem,set_cell_type(cell),spacegr)
!
!      Calcola distanza minime
       !call copy_atoms(atmc,atm)
       !call frac_to_cart(atmc,orthomatrix(cell))
       allocate(distminv(new_peaks),kminv(new_peaks))
       if (nat_old > 0) then
           do i=1,new_peaks
              iat = nat_old+i
!!!!test
          call get_minimum_distance_sym(atm(iat),atm(:nat_old),cell,spg,kminv(i),distminv(i),xeq)
          atm(iat)%xc = xeq
          !write(0,*)'At=',trim(atm(iat)%lab),distmin,trim(atm(kmin)%lab)
!!!!test
          !    call get_minimum_distance(atmc(iat),atmc(:nat_old),kmin,distmin)
          !    distminv(i) = distmin
          !    kminv(i) = kmin          !!mr
              !write(It(i),'(a,i0,4x,a,f0.3)')trim(atm(iat)%lab)//'  I=', nint(atm(iat)%inte),  &
              write(It(i),'(a,f8.2,4x,a,f0.3)')trim(atm(iat)%lab)//'  I=', atm(iat)%inte,  &
                'dmin('//trim(atm(iat)%lab)//'-'//trim(atm(kminv(i))%lab)//')=' ,distminv(i)
               It(i) = trim(It(i))//char(0)
           enddo
       else
           do i=1,new_peaks
              distminv(i) = 0
              kminv(i) = i
              !write(It(i),'(a,i0,4x,a,f0.3)')trim(atm(i)%lab)//'  I=', nint(atm(i)%inte),  &
              write(It(i),'(a,f8.2,4x,a,f0.3)')trim(atm(i)%lab)//'  I=', atm(i)%inte,  &
                'dmin('//trim(atm(i)%lab)//'-'//' - '//')=' ,distminv(i)
               It(i) = trim(It(i))//char(0)
           enddo
       endif
!
!      Disegna il modello e i nuovi atomi come ghost
       if (.not.jav_opened())call ilmview_open(StructureName)
       call jav_set_modify(0)
       call ilmview_draw(atm,lconn,celem,cell,spg)
!
       call ClearMessage()
! --
       msg = trim(i_to_s(new_peaks))//' new peaks found.'//char(10)//  &
             'Check Intensities &amp; distances and select peaks to retain.'//char(0)
       NumCan = new_peaks
       MultiSel = 1
       allocate (selected(NumCan))
       selected(:) = 0
       call c_NewPeekList('Select peaks to accept'//char(0), new_peaks,     &
                          MultiSel, selected,iExitC, nat_old, msg, atm, distminv, kminv)
       call jav_set_modify(1)           !!mr 13/01/2016
       if (iExitC == 3) then
           selected(:) = 0
       elseif (iExitC.eq.2) then
           selected(:) = 1
       endif
       Naccept = 0
       do i=1, NumCan
          if (selected(i).eq.1) then
              Naccept = Naccept + 1
              atm(nat_old+Naccept) = atm(nat_old+i)
          endif
       enddo
       if (Naccept == 0) ier = 1 ! segnala che non c'e' modifica
       call resize_atoms(atm,nat_old+Naccept)
       call ilmview_draw(atm,lconn,celem,cell,spg)
       call jav_set_modify(1)
!Gianluca 13/01/2016
      iAction = 2
      call ClessidraOnOff(iAction)
!Gianluca 13/01/2016
   else
       ier = -1
       call OpenWarning('No new peaks found !!!')
!!mr 5/06/2016
       call jav_set_modify(1)
       iAction = 2
       call ClessidraOnOff(iAction)
!!end mr
   endif
   call atom_to_xyzs(atm)
!Gianluca 13/01/2016
!         iAngolo = 0
!         leggi = 1
!         call toMolPlo3(leggi)
          call toMolPlo3(0)
          call RotazTot
          call GetBonds(1)
!Gianluca 13/01/2016
   !call print_connect(atm%lab,legm=lconn,kpri=0)
!
   end subroutine labelx

END MODULE fouriermap
