/*****************************************************************************/
/* struct mr_action_entry tipo di item: 
                  1 - tipico bottone della toolbar
                  2 - radio
                  3 - check
                  4 - separatore
                  5 - spin
                  6 - scale
                  7 - bottone (testo e immagine) rettangolare allineato a destra
                  8 - bottone (testo e immagine) rettangolare 
                  9 - bottone animato 
                 12 - label
                 13 - campo di entry
                 47 - event_box con pixmap 
                 50 - pixmap
                 52 - label boxed
                1xx - item di tipo xx (tra i precedenti) che apre una 
                      nuova toolbar
                -xx - item di tipo xx (tra i precedenti) da non mostrare
*/
/*****************************************************************************/

#include "callbacks.h"

gboolean ifToolbar = TRUE;
gboolean ifStatusbar = TRUE;

static mr_action_entry tb_entries[] = {
{0, 1, "Exit", "Exit", "Leave ", "quit", G_CALLBACK(exit_confirm), NULL, -1, 1, 0},
{0, 1, "New", "New", "New Project", "filenew1", G_CALLBACK(open_newproj_dlg), NULL, -1, 1, 0},
{0, 1, "Load", "Load", "Load input (.sir) file and start the run", "opensir", G_CALLBACK(open_file), NULL, 0, 1, 0},
{0, 1, "Old Project", "Old Project", "Load Project (.bin) file", "openbin", G_CALLBACK(open_file), NULL, 1, 1, 0},
{0, 1, "Save", "Save", "Save project file", "save", G_CALLBACK(c_savebinfile), NULL, -1, 0, 0},
{0, 1, "Restore", "Restore", "Restore project file", "restore", G_CALLBACK(c_restorebinfile), NULL, -1, 1, 0},
{0, 4},
{0, 1, "Reset view", "Reset View", "Reset the View", "reset", G_CALLBACK(view_options), NULL, 18, 0, 0},
{0, 3, "LabTog", "LabTog", "Labels On/Off", "labon", G_CALLBACK(toggle_label), NULL, -1, 0, 0},
{0, 1, "Plot Style TB", "Plot _Style", "Change the plot style of selected atoms", "plot_ico", G_CALLBACK(open_cplotstyle_dlg), NULL, -1, 0, 0},
{0, 1, "View", "Jav", "Open structure in Jav", "jav", G_CALLBACK(call_javs), NULL, -1, 0, 0},
{0, 1, "Map viewer", "contour", "Map viewer", "contour", G_CALLBACK(call_jav), NULL, -1, 0, 0},
{0, 1, "contour", "contour", "2D contour", "2D_contour", G_CALLBACK(mostra_tb), NULL, -1, 0, 0},
{0, -1, "terminal", "terminal", "interactive shell", "terminal", G_CALLBACK(open_terminal), NULL, -1, 0, 0}
};

static mr_action_entry tb_four[] = {
{0, 50, "iconFour", NULL, "Info", "fourier", NULL, NULL, -1, 1, 0},
{0, 12, "labelManage new peaks ", "Manage new peaks ", "label", NULL, NULL, NULL, -1, 1, 0},
{0, 4},
{0, 101, "FourDist", "FourDist", "Info on selected atoms", "who", G_CALLBACK(mostra_tbg1), NULL, -1, 1, 0},
{0, 1, "Plot Style", "Plot Style", "Plot Style", "plot_ico", G_CALLBACK(open_cplotstyle_dlg), NULL, -1, 1, 0},
{0, 3, "LabTogF", "LabTogF", "Labels On/Off", "labon", G_CALLBACK(toggle_label), NULL, -1, 1, 0},
{0, 101, "Change species", "Change species", "Change species", "relabel", G_CALLBACK(mostra_tbg1), NULL, -1, 1, 0},
{0, 80, "expanderFour", NULL, "expander", "void", NULL, NULL, 0, 0, 0},
{0, 7, "Back", "Back", "Back to Main Toolbar.", "back", G_CALLBACK(ok_tbfour), NULL, 0, 1, 0}
};

static mr_action_entry tb_idro[] = {
{0, 50, "iconIdro", NULL, "Info", "fourier", NULL, NULL, -1, 1, 0},
{0, 12, "Manage new peaks ", "Manage new peaks ", "label", NULL, NULL, NULL, -1, 1, 0},
{0, 4},
{0, 101, "FourDist", "FourDist", "Info on selected atoms", "who", G_CALLBACK(mostra_tbg1), NULL, -1, 1, 0},
{0, 1, "Plot Style", "Plot Style", "Plot Style", "plot_ico", G_CALLBACK(open_cplotstyle_dlg), NULL, -1, 1, 0},
{0, 3, "LabTogH", "LabTogH", "Labels On/Off", "labon", G_CALLBACK(toggle_label), NULL, -1, 1, 0},
{0, 101, "Accept hydrogen", "Accept hydrogen", "Accept hydrogen", "ok_idrogeni", G_CALLBACK(mostra_tbg1), NULL, -1, 1, 0},
{0, 80, "expanderHydro", NULL, "expander", "void", NULL, NULL, 0, 0, 0},
{0, 7, "Back", "Back", "Back to Main Toolbar.", "back", G_CALLBACK(ok_tbfour), NULL, 1, 1, 0}
};

static mr_action_entry tb_contour[] = {
{0, 50, "iconCont", NULL, "Info", "2D_contour", NULL, NULL, -1, 1, 0},
{0, 12, "labelCont", "Map contour", "label", NULL, NULL, NULL, -1, 1, 0},
{0, 4},
{0, 1, "Set Levels", "Set Levels", "Set Levels", "levels", G_CALLBACK(open_cont_dlg), NULL, 1, 1, 0},
{0, 1, "Set Limits", "Set Limits", "Set Limits & drawing Resolution", "contour_limiti", G_CALLBACK(open_cont_dlg), NULL, 2, 1, 0},
{0, 1, "ModelCont", "Model", "Model On/Off", "modelcont", G_CALLBACK(open_model_cont), NULL, -1, 1, 0},
{0, 3, "CellOnC", "CellOnC", "Cell On/Off", "cellon", G_CALLBACK(toggle_cell), NULL, -1, 1, 0},
{0, 4},
{0, 1, "Zoom In", "Zoom In", "Zoom In", "zoom_in", G_CALLBACK(view_options), NULL, 15, 1, 0},
{0, 1, "Zoom Out", "Zoom Out", "Zoom Out", "zoom_out", G_CALLBACK(view_options), NULL, 16, 1, 0},
{0, 1, "Reset View", "Reset View", "Reset the View", "reset", G_CALLBACK(view_options), NULL, 17, 1, 0},
{0, 4},
{0, 1, "Normal to a", NULL, "Normal to a", "alonga", G_CALLBACK(view_options), NULL, 21, 1, 0},
{0, 1, "Normal to b", NULL, "Normal to b", "alongb", G_CALLBACK(view_options), NULL, 22, 1, 0},
{0, 1, "Normal to c", NULL, "Normal to c", "alongc", G_CALLBACK(view_options), NULL, 23, 1, 0},
{0, 1, "Previous Section", "Previous Section", "Previous section", "prev_layer", G_CALLBACK(change_section), NULL, 0, 1, 0},
{0, 1, "Next Section", "Next Section", "Next section", "next_layer", G_CALLBACK(change_section), NULL, 2, 1, 0},
{0, 1, "Multi Section", "Multi Section", "Display many sections", "multi_layer", G_CALLBACK(open_cont_dlg), NULL, 3, 1, 0},
{0, 3, "Grid On/Off", "Grid On/Off", "Grid On/Off", "gridon", G_CALLBACK(toggle_grid), NULL, -1, 1, 0},
{0, 4},
{0, 1, "InfoCont", "Info", "About the Contour routine", "info_about", G_CALLBACK(about_contour), NULL, -2, 1, 0},
{0, 80, "expanderCont", NULL, "expander", "void", NULL, NULL, 0, 0, 0},
{0, 7, "Back", "Back", "Back to Main Toolbar.", "back", G_CALLBACK(ok_tbcontour), NULL, -1, 1, 0}
};

static mr_action_entry tb_run[] = {
{0, 50, "iconRun", "icon", NULL, "go", NULL, NULL, -1, 1, 0},
{0, 4},
{0, 12, "labelRun", "Info: ", "label", NULL, NULL, NULL, -1, 1, 0},
{0, 52, "entry", "entry", NULL, NULL, NULL, NULL, -1, 1, 0},
{0, 80, "expanderRun", NULL, "expander", "void", NULL, NULL, 0, 0, 0},
//{0, 7, "Stop", "Stop", "Stop the run", "stopr", G_CALLBACK(stop_run), NULL, 1, 0, 0},
//{0, -7, "Stopping", "Stopping", "Stopping the run", "ystopr", NULL, NULL, 1, 0, 0}
};

static mr_action_entry tb_g1[] = {
{0, 50, "iconG1", "icon", NULL, "void", NULL, NULL, -1, 1, 0},
{0, 4},
{0, 12, "labelG1", "label", NULL, NULL, NULL, NULL, -1, 1, 0},
{0, 13, "entryG1", "entry", NULL, NULL, NULL, NULL, -1, 1, 0},
{0, 5, "spinG1", "spin", NULL, "3, 1.5, 0.85, 4, 0.05, 1", G_CALLBACK(on_sfrad_changed), NULL, -1, 1, 0},
{0, 50, "iconG1b", "icon", NULL, "void", NULL, NULL, -1, 0, 0},
{0, 80, "expanderG1", NULL, "expander", "void", NULL, NULL, 0, 0, 0},
{0, 47, "icon_cest", "icon_cest", "Click on safe to restore hidden labels", "cassaforte_chiusa,cassaforte_aperta.png", G_CALLBACK(trashcan), NULL, -1, 1, 0},
{0, 7, "Close Polygon", "Close Polygon", "Close Polygon", "closep", G_CALLBACK(ok_tbselect), NULL, -1, 1, 0},
{0, 7, "Ok", "Ok", "OK", "ok", G_CALLBACK(ok_tb1), NULL, -1, 1, 0},
{0, 7, "BackG1", "Back", "Back to previous toolbar", "back", G_CALLBACK(cancel_tb), NULL, -1, 1, 0}
};

static mr_action_entry tb_run2[] = {
{0, 50, "iconRun2", "icon", NULL, "go", NULL, NULL, -1, 1, 0},
{0, 4},
{0, 12, "labelRun2", "Info: ", "label", NULL, NULL, NULL, -1, 1, 0},
{0, 52, "entryRun2", "entry", NULL, NULL, NULL, NULL, -1, 1, 0},
{0, 80, "expander", NULL, "expander", "void", NULL, NULL, 0, 0, 0},
{0, 1, "jav", "jav", "open viewer", "viewmol", G_CALLBACK(call_javs), NULL, 11, 1, 0},
{0, 7, "Skip", "Skip", "Skip to the next run", "nextgr", G_CALLBACK(skip_or_stop), NULL, 1, 1, 0},
{0, 7, "SkiporStop", "Stop", "Stop the procedure", "stopr", G_CALLBACK(skip_or_stop), NULL, 2, 1, 0}
};
static const guint n_entries[] = {
                                   G_N_ELEMENTS(tb_entries),
                                   G_N_ELEMENTS(tb_four),
                                   G_N_ELEMENTS(tb_idro),
                                   G_N_ELEMENTS(tb_contour),
                                   G_N_ELEMENTS(tb_run),
                                   G_N_ELEMENTS(tb_g1),
                                   G_N_ELEMENTS(tb_run2)};

static gpointer toolbaritems[] = {(gpointer) tb_entries,
                                   (gpointer) tb_four,
                                   (gpointer) tb_idro,
                                   (gpointer) tb_contour,
                                   (gpointer) tb_run,
                                   (gpointer) tb_g1,
                                   (gpointer) tb_run2};

static const char *Nomi[] =  { "toolbar_main",
                               "toolbar_four",
                               "toolbar_idro",
                               "toolbar_contour",
                               "toolbar_run",
                               "toolbar_g1",
                               "toolbar_run2"};

#ifdef USE_ACTIONS
static const gchar *tbui_info[] =
{
"<ui>"
"  <toolbar name='toolbar_main'>"
"    <toolitem action='Exit'/>"
"    <toolitem action='New'/>"
"    <toolitem action='Load'/>"
"    <toolitem action='Old Project'/>"
"    <toolitem action='Save'/>"
"    <toolitem action='Restore'/>"
"    <separator/>"
"    <toolitem action='LabTog'/>"
"    <toolitem action='Reset view'/>"
"    <toolitem action='Plot Style TB'/>"
"    <toolitem action='View'/>"
"    <toolitem action='Map viewer'/>"
"    <toolitem action='contour'/>"
"    <toolitem action='terminal'/>"
"  </toolbar>"
"</ui>",
"<ui>"
"  <toolbar name='toolbar_four'>"
"    <toolitem action='iconFour'/>"
"    <toolitem action='labelManage new peaks '/>"
"    <separator/>"
"    <toolitem action='FourDist'/>"
"    <toolitem action='Plot Style'/>"
"    <toolitem action='LabTogF'/>"
"    <toolitem action='Change species'/>"
"    <toolitem action='expanderFour'/>"
"    <toolitem action='Back'/>"
"  </toolbar>"
"</ui>",
"<ui>"
"  <toolbar name='toolbar_idro'>"
"    <toolitem action='iconIdro'/>"
"    <toolitem action='Manage new peaks '/>"
"    <separator/>"
"    <toolitem action='FourDist'/>"
"    <toolitem action='Plot Style'/>"
"    <toolitem action='LabTogH'/>"
"    <toolitem action='Accept hydrogen'/>"
"    <toolitem action='expanderHydro'/>"
"    <toolitem action='Back'/>"
"  </toolbar>"
"</ui>",
"<ui>"
"  <toolbar name='toolbar_contour'>"
"    <toolitem action='iconCont'/>"
"    <toolitem action='labelCont'/>"
"    <separator/>"
"    <toolitem action='Set Levels'/>"
"    <toolitem action='Set Limits'/>"
"    <toolitem action='ModelCont'/>"
"    <toolitem action='CellOnC'/>"
"    <separator/>"
"    <toolitem action='Zoom In'/>"
"    <toolitem action='Zoom Out'/>"
"    <toolitem action='Reset View'/>"
"    <separator/>"
"    <toolitem action='Normal to a'/>"
"    <toolitem action='Normal to b'/>"
"    <toolitem action='Normal to c'/>"
"    <toolitem action='Previous Section'/>"
"    <toolitem action='Next Section'/>"
"    <toolitem action='Multi Section'/>"
"    <toolitem action='Grid On/Off'/>"
"    <separator/>"
"    <toolitem action='InfoCont'/>"
"    <toolitem action='expanderCont'/>"
"    <toolitem action='Back'/>"
"  </toolbar>"
"</ui>",
"<ui>"
"  <toolbar name='toolbar_run'>"
"    <toolitem action='iconRun'/>"
"    <separator/>"
"    <toolitem action='labelRun'/>"
"    <toolitem action='entry'/>"
"    <toolitem action='expanderRun'/>"
"  </toolbar>"
"</ui>",
"<ui>"
"  <toolbar name='toolbar_g1'>"
"    <toolitem action='iconG1'/>"
"    <separator/>"
"    <toolitem action='labelG1'/>"
"    <toolitem action='entryG1'/>"
"    <toolitem action='spinG1'/>"
"    <toolitem action='iconG1b'/>"
"    <toolitem action='expanderG1'/>"
"    <toolitem action='icon_cest'/>"
"    <toolitem action='Close Polygon'/>"
"    <toolitem action='Ok'/>"
"    <toolitem action='BackG1'/>"
"  </toolbar>"
"</ui>",
"<ui>"
"  <toolbar name='toolbar_run2'>"
"    <toolitem action='iconRun2'/>"
"    <separator/>"
"    <toolitem action='labelRun2'/>"
"    <toolitem action='entryRun2'/>"
"    <toolitem action='expander'/>"
"    <toolitem action='jav'/>"
"    <toolitem action='Skip'/>"
"    <toolitem action='SkiporStop'/>"
"  </toolbar>"
"</ui>"
};
#endif
